#  Copyright 2019 Shawn Seymour. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License"). You
#  may not use this file except in compliance with the License. A copy of
#  the License is located at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  or in the "license" file accompanying this file. This file is
#  distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
#  ANY KIND, either express or implied. See the License for the specific
#  language governing permissions and limitations under the License.

from __future__ import unicode_literals

import logging
import signal
import sys
from functools import partial

from kafka_healthcheck import handler
from kafka_healthcheck import health
from kafka_healthcheck import parser

try:
    from SocketServer import TCPServer as HTTPServer
except ImportError:
    from http.server import HTTPServer


def main():
    config_parser = parser.get_parser()
    args = config_parser.parse_args()

    logging.basicConfig(format="%(asctime)s.%(msecs)03d [%(levelname)7s] - %(message)s",
                        datefmt="%Y-%m-%d %H:%M:%S",
                        level=args.log_level)

    health_object = health.Health(args.kafka_host, args.kafka_port)
    request_handler = partial(handler.RequestHandler, health_object)

    HTTPServer.allow_reuse_address = True
    httpd = HTTPServer(("", args.healthcheck_port), request_handler)
    logging.info("Serving requests at http://localhost:{}".format(args.healthcheck_port))

    def stop(status_code, frame):
        logging.info("SIGINT/SIGTERM; exiting...")
        httpd.server_close()
        sys.exit(0)

    signal.signal(signal.SIGINT, stop)
    signal.signal(signal.SIGTERM, stop)

    try:
        httpd.serve_forever()
    except KeyboardInterrupt:
        stop(0, None)


if __name__ == "__main__":
    main()



#  Copyright 2019 Shawn Seymour. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License"). You
#  may not use this file except in compliance with the License. A copy of
#  the License is located at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  or in the "license" file accompanying this file. This file is
#  distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
#  ANY KIND, either express or implied. See the License for the specific
#  language governing permissions and limitations under the License.

__version__ = "0.0.1"


def get_version():
    return __version__



#  Copyright 2019 Shawn Seymour. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License"). You
#  may not use this file except in compliance with the License. A copy of
#  the License is located at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  or in the "license" file accompanying this file. This file is
#  distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
#  ANY KIND, either express or implied. See the License for the specific
#  language governing permissions and limitations under the License.

from __future__ import unicode_literals

import json

try:
    from BaseHTTPServer import BaseHTTPRequestHandler
except ImportError:
    from http.server import BaseHTTPRequestHandler


class RequestHandler(BaseHTTPRequestHandler):

    def __init__(self, health, *args, **kwargs):
        self.health = health
        BaseHTTPRequestHandler.__init__(self, *args, **kwargs)

    def do_GET(self):
        health_result = self.health.get_health_result()
        health_result_json = json.dumps(health_result)
        self.send_response(200 if health_result["healthy"] is True else 503)
        self.send_header("content-type", "application/json")
        self.end_headers()
        self.wfile.write(bytes(health_result_json.encode("utf-8")))

    def log_message(self, format, *args):
        return



#  Copyright 2019 Shawn Seymour. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License"). You
#  may not use this file except in compliance with the License. A copy of
#  the License is located at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  or in the "license" file accompanying this file. This file is
#  distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
#  ANY KIND, either express or implied. See the License for the specific
#  language governing permissions and limitations under the License.

from __future__ import unicode_literals

import logging
import subprocess


class Health:

    def __init__(self, kafka_host, kafka_port):
        self.kafka_host = kafka_host
        self.kafka_port = kafka_port
        self.log_initialization_values()

    def get_health_result(self):
        try:
            is_healthy = self.is_healthy()
            health_result = {"healthy": is_healthy}
        except Exception as ex:
            logging.error("Error while attempting to calculate health result. Assuming unhealthy. Error: {}".format(ex))
            health_result = {
                "healthy": False,
                "message": "Exception raised while attempting to calculate health result, assuming unhealthy.",
                "error": "{}".format(ex)
            }
        return health_result

    def log_initialization_values(self):
        logging.info("Server will healthcheck against kafka host: {}".format(self.kafka_host))
        logging.info("Server will healthcheck against kafka port: {}".format(self.kafka_port))

    def is_healthy(self):
        process_one = subprocess.Popen(
            ["kcat", "-b", "{}:{}".format(self.kafka_host, self.kafka_port), "-L"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE)
        (output, err) = process_one.communicate()
        exit_code = process_one.wait()
        output = output.decode("utf-8").strip()
        if exit_code == 0:
            logging.debug("kafkacat returned response: {}".format(output))
            is_ok = output.startswith("Metadata for all topics")
            if is_ok:
                logging.debug("kafka Metadata check returned response: {}".format(output))
            else:
                logging.warning("kafka Metadata is not healthy: {}".format(output))
            return is_ok
        else:
            logging.warning("kafka Metadata returned exit code: {}, marking unhealthy...".format(exit_code))
            return False



#  Copyright 2019 Shawn Seymour. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License"). You
#  may not use this file except in compliance with the License. A copy of
#  the License is located at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  or in the "license" file accompanying this file. This file is
#  distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
#  ANY KIND, either express or implied. See the License for the specific
#  language governing permissions and limitations under the License.

import argparse
import os


def get_parser():
    parser = argparse.ArgumentParser(description="A simple healthcheck for kafka. It checks if kafka is 'ok' via 'kafkacat'")

    parser.add_argument("--port",
                        default=os.environ.get("HEALTHCHECK_PORT", "9290"),
                        dest="healthcheck_port",
                        type=int,
                        nargs="?",
                        help="The port for the healthcheck HTTP server."
                        )

    parser.add_argument("--kafka-host",
                        default=os.environ.get("HEALTHCHECK_KAFKA_HOST", "localhost"),
                        dest="kafka_host",
                        nargs="?",
                        help="The host of the Kafka instance that the health check will be run against."
                        )

    parser.add_argument("--kafka-port",
                        default=os.environ.get("HEALTHCHECK_KAFKA_PORT", "9092"),
                        dest="kafka_port",
                        type=int,
                        nargs="?",
                        help="The port of the Kafka instance that the health check will be run against."
                        )

    parser.add_argument("--log-level",
                        default=os.environ.get("HEALTHCHECK_LOG_LEVEL", "INFO").upper(),
                        dest="log_level",
                        nargs="?",
                        help="The level of logs to be shown. Default: INFO.")
    return parser



import setuptools

meta = {}

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("kafka_healthcheck/version.py") as f:
    exec(f.read(), meta)

requires = [
]

setuptools.setup(
    name="kafka-healthcheck",
    version=meta["__version__"],
    author="Rodrigo Garcia fork of Shawn Seymour",
    author_email="rodrigo.garcia@despgar.com, shawn@devshawn.com",
    description="A simple healthcheck wrapper to monitor kafka with kafkacat.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/rodrigongarcia/kafka-healthcheck",
    license="Apache License 2.0",
    packages=["kafka_healthcheck"],
    install_requires=requires,
    entry_points={
        "console_scripts": ["kafka-healthcheck=kafka_healthcheck.main:main"],
    },
    keywords=("kafka", "health", "healthcheck", "wrapper", "monitor"),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Operating System :: Unix",
        "Programming Language :: Python :: 3.9",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
